@@ -1,15 +1,16 @@
 use crate::annis::db::exec::nodesearch::NodeSearchSpec;
 use crate::annis::db::AnnotationStorage;
-use crate::AnnotationGraph;
 use crate::{
     annis::{
         db::aql::model::{AnnotationComponentType, TOK, TOKEN_KEY},
         operator::*,
     },
+    errors::Result,
     graph::Match,
 };
+use crate::{try_as_boxed_iter, AnnotationGraph};
 use graphannis_core::{
-    annostorage::{MatchGroup, ValueSearch},
+    annostorage::ValueSearch,
     graph::ANNIS_NS,
     types::{Component, NodeID},
 };
@@ -33,8 +34,8 @@ impl BinaryOperatorSpec for EqualValueSpec {
         HashSet::default()
     }
 
-    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Option<BinaryOperator<'a>> {
-        Some(BinaryOperator::Index(Box::new(EqualValue {
+    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Result<BinaryOperator<'a>> {
+        Ok(BinaryOperator::Index(Box::new(EqualValue {
             node_annos: db.get_node_annos(),
             spec_left: self.spec_left.clone(),
             spec_right: self.spec_right.clone(),
@@ -74,22 +75,24 @@ impl<'a> std::fmt::Display for EqualValue<'a> {
 }
 
 impl<'a> EqualValue<'a> {
-    fn value_for_match(&self, m: &Match, spec: &NodeSearchSpec) -> Option<Cow<str>> {
+    fn value_for_match(&self, m: &Match, spec: &NodeSearchSpec) -> Result<Option<Cow<str>>> {
         match spec {
             NodeSearchSpec::ExactValue { .. }
             | NodeSearchSpec::NotExactValue { .. }
             | NodeSearchSpec::RegexValue { .. }
             | NodeSearchSpec::NotRegexValue { .. } => {
-                self.node_annos.get_value_for_item(&m.node, &m.anno_key)
+                let val = self.node_annos.get_value_for_item(&m.node, &m.anno_key)?;
+                Ok(val)
             }
             NodeSearchSpec::AnyToken
             | NodeSearchSpec::ExactTokenValue { .. }
             | NodeSearchSpec::NotExactTokenValue { .. }
             | NodeSearchSpec::RegexTokenValue { .. }
             | NodeSearchSpec::NotRegexTokenValue { .. } => {
-                self.node_annos.get_value_for_item(&m.node, &TOKEN_KEY)
+                let val = self.node_annos.get_value_for_item(&m.node, &TOKEN_KEY)?;
+                Ok(val)
             }
-            NodeSearchSpec::AnyNode => None,
+            NodeSearchSpec::AnyNode => Ok(None),
         }
     }
 
@@ -116,11 +119,11 @@ impl<'a> EqualValue<'a> {
 }
 
 impl<'a> BinaryOperatorBase for EqualValue<'a> {
-    fn filter_match(&self, lhs: &Match, rhs: &Match) -> bool {
-        let lhs_val = self.value_for_match(lhs, &self.spec_left);
-        let rhs_val = self.value_for_match(rhs, &self.spec_right);
+    fn filter_match(&self, lhs: &Match, rhs: &Match) -> Result<bool> {
+        let lhs_val = self.value_for_match(lhs, &self.spec_left)?;
+        let rhs_val = self.value_for_match(rhs, &self.spec_right)?;
 
-        if let (Some(lhs_val), Some(rhs_val)) = (lhs_val, rhs_val) {
+        let result = if let (Some(lhs_val), Some(rhs_val)) = (lhs_val, rhs_val) {
             if self.negated {
                 lhs_val != rhs_val
             } else {
@@ -128,13 +131,14 @@ impl<'a> BinaryOperatorBase for EqualValue<'a> {
             }
         } else {
             false
-        }
+        };
+        Ok(result)
     }
 
-    fn estimation_type(&self) -> EstimationType {
+    fn estimation_type(&self) -> Result<EstimationType> {
         if let Some((ns, name)) = EqualValue::anno_def_for_spec(&self.spec_left) {
             if let Some(most_frequent_value_left) =
-                self.node_annos.guess_most_frequent_value(ns, name)
+                self.node_annos.guess_most_frequent_value(ns, name)?
             {
                 if let Some((ns, name)) = EqualValue::anno_def_for_spec(&self.spec_right) {
                     let guessed_count_right = self.node_annos.guess_max_count(
@@ -142,36 +146,41 @@ impl<'a> BinaryOperatorBase for EqualValue<'a> {
                         name,
                         &most_frequent_value_left,
                         &most_frequent_value_left,
-                    );
+                    )?;
 
-                    let total_annos = self.node_annos.number_of_annotations_by_name(ns, name);
+                    let total_annos = self.node_annos.number_of_annotations_by_name(ns, name)?;
                     let sel = guessed_count_right as f64 / total_annos as f64;
                     if self.negated {
-                        return EstimationType::Selectivity(1.0 - sel);
+                        return Ok(EstimationType::Selectivity(1.0 - sel));
                     } else {
-                        return EstimationType::Selectivity(sel);
+                        return Ok(EstimationType::Selectivity(sel));
                     }
                 }
             }
         }
         // fallback to default
-        EstimationType::Selectivity(0.5)
+        Ok(EstimationType::Selectivity(0.5))
     }
 
-    fn get_inverse_operator<'b>(&self, graph: &'b AnnotationGraph) -> Option<BinaryOperator<'b>> {
-        Some(BinaryOperator::Index(Box::from(EqualValue {
+    fn get_inverse_operator<'b>(
+        &self,
+        graph: &'b AnnotationGraph,
+    ) -> Result<Option<BinaryOperator<'b>>> {
+        let inverse = BinaryOperator::Index(Box::from(EqualValue {
             node_annos: graph.get_node_annos(),
             spec_left: self.spec_left.clone(),
             spec_right: self.spec_right.clone(),
             negated: self.negated,
-        })))
+        }));
+        Ok(Some(inverse))
     }
 }
 
 impl<'a> BinaryOperatorIndex for EqualValue<'a> {
-    fn retrieve_matches(&self, lhs: &Match) -> Box<dyn Iterator<Item = Match>> {
+    fn retrieve_matches<'b>(&'b self, lhs: &Match) -> Box<dyn Iterator<Item = Result<Match>> + 'b> {
         let lhs = lhs.clone();
-        if let Some(lhs_val) = self.value_for_match(&lhs, &self.spec_left) {
+        let lhs_val = try_as_boxed_iter!(self.value_for_match(&lhs, &self.spec_left));
+        if let Some(lhs_val) = lhs_val {
             let val_search: ValueSearch<&str> = if self.negated {
                 ValueSearch::NotSome(&lhs_val)
             } else {
@@ -179,11 +188,11 @@ impl<'a> BinaryOperatorIndex for EqualValue<'a> {
             };
 
             if let Some((ns, name)) = EqualValue::anno_def_for_spec(&self.spec_right) {
-                let rhs_candidates: MatchGroup = self
+                let rhs_candidates = self
                     .node_annos
                     .exact_anno_search(ns, name, val_search)
-                    .collect();
-                return Box::new(rhs_candidates.into_iter());
+                    .map(|m| m.map_err(|e| e.into()));
+                return Box::new(rhs_candidates);
             }
         }
         Box::new(std::iter::empty())
