@@ -1,19 +1,23 @@
 use crate::annis::db::token_helper;
 use crate::annis::db::token_helper::TokenHelper;
+use crate::annis::errors::GraphAnnisError;
 use crate::annis::operator::{BinaryOperator, BinaryOperatorIndex, EstimationType};
-use crate::AnnotationGraph;
 use crate::{
     annis::operator::{BinaryOperatorBase, BinaryOperatorSpec},
+    errors::Result,
     graph::{GraphStorage, Match},
     model::AnnotationComponentType,
 };
+use crate::{try_as_boxed_iter, AnnotationGraph};
+use graphannis_core::types::NodeID;
 use graphannis_core::{
     graph::{ANNIS_NS, DEFAULT_ANNO_KEY},
     types::Component,
 };
+use itertools::Itertools;
 
 use std::any::Any;
-use std::collections::{HashSet, VecDeque};
+use std::collections::HashSet;
 use std::sync::Arc;
 
 #[derive(Clone, Debug, PartialOrd, Ord, Hash, PartialEq, Eq)]
@@ -45,7 +49,7 @@ impl BinaryOperatorSpec for InclusionSpec {
         v
     }
 
-    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Option<BinaryOperator<'a>> {
+    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Result<BinaryOperator<'a>> {
         let optional_op = Inclusion::new(db);
         optional_op.map(|op| BinaryOperator::Index(Box::new(op)))
     }
@@ -60,12 +64,16 @@ impl BinaryOperatorSpec for InclusionSpec {
 }
 
 impl<'a> Inclusion<'a> {
-    pub fn new(db: &'a AnnotationGraph) -> Option<Inclusion<'a>> {
-        let gs_order = db.get_graphstorage(&COMPONENT_ORDER)?;
+    pub fn new(db: &'a AnnotationGraph) -> Result<Inclusion<'a>> {
+        let gs_order = db.get_graphstorage(&COMPONENT_ORDER).ok_or_else(|| {
+            GraphAnnisError::ImpossibleSearch(
+                "Ordering component missing (needed for _i_ operator)".to_string(),
+            )
+        })?;
 
         let tok_helper = TokenHelper::new(db)?;
 
-        Some(Inclusion {
+        Ok(Inclusion {
             gs_order,
             tok_helper,
         })
@@ -79,9 +87,9 @@ impl<'a> std::fmt::Display for Inclusion<'a> {
 }
 
 impl<'a> BinaryOperatorBase for Inclusion<'a> {
-    fn filter_match(&self, lhs: &Match, rhs: &Match) -> bool {
-        let left_right_lhs = self.tok_helper.left_right_token_for(lhs.node);
-        let left_right_rhs = self.tok_helper.left_right_token_for(rhs.node);
+    fn filter_match(&self, lhs: &Match, rhs: &Match) -> Result<bool> {
+        let left_right_lhs = self.tok_helper.left_right_token_for(lhs.node)?;
+        let left_right_rhs = self.tok_helper.left_right_token_for(rhs.node)?;
         if let (Some(start_lhs), Some(end_lhs), Some(start_rhs), Some(end_rhs)) = (
             left_right_lhs.0,
             left_right_lhs.1,
@@ -89,25 +97,25 @@ impl<'a> BinaryOperatorBase for Inclusion<'a> {
             left_right_rhs.1,
         ) {
             // span length of LHS
-            if let Some(l) = self.gs_order.distance(start_lhs, end_lhs) {
+            if let Some(l) = self.gs_order.distance(start_lhs, end_lhs)? {
                 // path between left-most tokens exists in ORDERING component and has maximum length l
-                if self.gs_order.is_connected(start_lhs, start_rhs, 0, std::ops::Bound::Included(l))
+                if self.gs_order.is_connected(start_lhs, start_rhs, 0, std::ops::Bound::Included(l))?
                 // path between right-most tokens exists in ORDERING component and has maximum length l
-                && self.gs_order.is_connected(end_rhs, end_lhs, 0, std::ops::Bound::Included(l))
+                && self.gs_order.is_connected(end_rhs, end_lhs, 0, std::ops::Bound::Included(l))?
                 {
-                    return true;
+                    return Ok(true);
                 }
             }
         }
 
-        false
+        Ok(false)
     }
 
     fn is_reflexive(&self) -> bool {
         false
     }
 
-    fn estimation_type(&self) -> EstimationType {
+    fn estimation_type(&self) -> Result<EstimationType> {
         if let (Some(stats_order), Some(stats_left)) = (
             self.gs_order.get_statistics(),
             self.tok_helper.get_gs_left_token().get_statistics(),
@@ -129,57 +137,88 @@ impl<'a> BinaryOperatorBase for Inclusion<'a> {
             }
             if sum_cov_nodes == 0 {
                 // only token in this corpus
-                return EstimationType::Selectivity(1.0 / num_of_token);
+                return Ok(EstimationType::Selectivity(1.0 / num_of_token));
             } else {
-                return EstimationType::Selectivity((sum_included as f64) / (sum_cov_nodes as f64));
+                return Ok(EstimationType::Selectivity(
+                    (sum_included as f64) / (sum_cov_nodes as f64),
+                ));
             }
         }
 
-        EstimationType::Selectivity(0.1)
+        Ok(EstimationType::Selectivity(0.1))
     }
 }
 
+enum NodeType {
+    Token(NodeID),
+    Other(NodeID),
+}
+
 impl<'a> BinaryOperatorIndex for Inclusion<'a> {
-    #[allow(clippy::needless_collect)]
-    fn retrieve_matches(&self, lhs: &Match) -> Box<dyn Iterator<Item = Match>> {
-        if let (Some(start_lhs), Some(end_lhs)) = self.tok_helper.left_right_token_for(lhs.node) {
+    fn retrieve_matches<'b>(&'b self, lhs: &Match) -> Box<dyn Iterator<Item = Result<Match>> + 'b> {
+        let left_right_token = try_as_boxed_iter!(self.tok_helper.left_right_token_for(lhs.node));
+        if let (Some(start_lhs), Some(end_lhs)) = left_right_token {
             // span length of LHS
-            if let Some(l) = self.gs_order.distance(start_lhs, end_lhs) {
+            let l = try_as_boxed_iter!(self.gs_order.distance(start_lhs, end_lhs));
+            if let Some(l) = l {
                 // find each token which is between the left and right border
-                let result: VecDeque<Match> = self
-                    .gs_order
-                    .find_connected(start_lhs, 0, std::ops::Bound::Included(l))
-                    .flat_map(move |t| {
-                        let it_aligned = self
+                let overlapped_token =
+                    self.gs_order
+                        .find_connected(start_lhs, 0, std::ops::Bound::Included(l));
+                // get the nodes that are covering these overlapped tokens
+                let candidates = overlapped_token
+                    .map_ok(move |t| {
+                        let others = self
                             .tok_helper
                             .get_gs_left_token()
                             .get_ingoing_edges(t)
-                            .filter(move |n| {
-                                // right-aligned token of candidate
+                            .map_ok(NodeType::Other);
+                        // return the token itself and all aligned nodes
+                        std::iter::once(Ok(NodeType::Token(t))).chain(others)
+                    })
+                    .flatten_ok()
+                    // Unwrap the Result<Result<_>>
+                    .map(|c| match c {
+                        Ok(c) => match c {
+                            Ok(c) => Ok(c),
+                            Err(e) => Err(GraphAnnisError::from(e)),
+                        },
+                        Err(e) => Err(GraphAnnisError::from(e)),
+                    });
+                // we need to check if the the RHS of these candidates is also included by the original span
+                let result = candidates
+                    .map(move |n| {
+                        let n = n?;
+                        match n {
+                            NodeType::Token(t) => Ok(Some(t)),
+                            NodeType::Other(n) => {
+                                // get right-aligned token of candidate
                                 let mut end_n =
-                                    self.tok_helper.get_gs_right_token_().get_outgoing_edges(*n);
+                                    self.tok_helper.get_gs_right_token_().get_outgoing_edges(n);
                                 if let Some(end_n) = end_n.next() {
-                                    // path between right-most tokens exists in ORDERING component
-                                    // and has maximum length l
-                                    self.gs_order.is_connected(
+                                    let end_n = end_n?;
+                                    if self.gs_order.is_connected(
                                         end_n,
                                         end_lhs,
                                         0,
                                         std::ops::Bound::Included(l),
-                                    )
-                                } else {
-                                    false
+                                    )? {
+                                        // path between right-most tokens exists in ORDERING component
+                                        // and has maximum length l
+                                        return Ok(Some(n));
+                                    }
                                 }
-                            });
-                        // return the token itself and all aligned nodes
-                        std::iter::once(t).chain(it_aligned)
+                                Ok(None)
+                            }
+                        }
                     })
-                    .map(|n| Match {
+                    // Only include the ones where the constraint was met
+                    .filter_map_ok(|n| n)
+                    .map_ok(|n| Match {
                         node: n,
                         anno_key: DEFAULT_ANNO_KEY.clone(),
-                    })
-                    .collect();
-                return Box::new(result.into_iter());
+                    });
+                return Box::new(result);
             }
         }
 
