@@ -132,31 +132,21 @@ impl KeySerializer for TextProperty {
         result
     }
 
-    fn parse_key(key: &[u8]) -> Self {
+    fn parse_key(
+        key: &[u8],
+    ) -> std::result::Result<Self, Box<dyn std::error::Error + Send + Sync>> {
         let id_size = std::mem::size_of::<u32>();
         let mut id_offset = key.len() - id_size * 3;
         let key_as_string: std::string::String = std::string::String::from_utf8_lossy(key).into();
         let segmentation_vector: Vec<_> = key_as_string.split_terminator('\0').collect();
 
-        let corpus_id = u32::from_be_bytes(
-            key[id_offset..(id_offset + id_size)]
-                .try_into()
-                .expect("TextProperty deserialization key was too small"),
-        );
+        let corpus_id = u32::from_be_bytes(key[id_offset..(id_offset + id_size)].try_into()?);
         id_offset += id_size;
 
-        let text_id = u32::from_be_bytes(
-            key[id_offset..(id_offset + id_size)]
-                .try_into()
-                .expect("TextProperty deserialization key was too small"),
-        );
+        let text_id = u32::from_be_bytes(key[id_offset..(id_offset + id_size)].try_into()?);
         id_offset += id_size;
 
-        let val = u32::from_be_bytes(
-            key[id_offset..(id_offset + id_size)]
-                .try_into()
-                .expect("TextProperty deserialization key was too small"),
-        );
+        let val = u32::from_be_bytes(key[id_offset..(id_offset + id_size)].try_into()?);
 
         let segmentation = if segmentation_vector.is_empty() {
             String::from("")
@@ -164,16 +154,16 @@ impl KeySerializer for TextProperty {
             segmentation_vector[0].into()
         };
 
-        TextProperty {
+        Ok(TextProperty {
             segmentation,
             corpus_id,
             text_id,
             val,
-        }
+        })
     }
 }
 
-#[derive(Clone, PartialEq, Eq, Hash, MallocSizeOf)]
+#[derive(Clone, PartialEq, Eq, Hash, MallocSizeOf, PartialOrd, Ord, Serialize, Deserialize)]
 struct TextKey {
     id: u32,
     corpus_ref: Option<u32>,
@@ -189,28 +179,22 @@ impl KeySerializer for TextKey {
         result
     }
 
-    fn parse_key(key: &[u8]) -> Self {
+    fn parse_key(
+        key: &[u8],
+    ) -> std::result::Result<Self, Box<dyn std::error::Error + Send + Sync>> {
         let id_size = std::mem::size_of::<u32>();
-        let id = u32::from_be_bytes(
-            key[0..id_size]
-                .try_into()
-                .expect("TextKey deserialization key was too small"),
-        );
+        let id = u32::from_be_bytes(key[0..id_size].try_into()?);
         let corpus_ref = if key.len() == id_size * 2 {
-            Some(u32::from_be_bytes(
-                key[id_size..]
-                    .try_into()
-                    .expect("TextKey deserialization key was too small"),
-            ))
+            Some(u32::from_be_bytes(key[id_size..].try_into()?))
         } else {
             None
         };
 
-        TextKey { id, corpus_ref }
+        Ok(TextKey { id, corpus_ref })
     }
 }
 
-#[derive(Clone, PartialEq, PartialOrd, Eq, Ord, MallocSizeOf)]
+#[derive(Clone, PartialEq, PartialOrd, Eq, Ord, MallocSizeOf, Serialize, Deserialize)]
 struct NodeByTextEntry {
     text_id: u32,
     corpus_ref: u32,
@@ -226,33 +210,23 @@ impl KeySerializer for NodeByTextEntry {
         result
     }
 
-    fn parse_key(key: &[u8]) -> Self {
+    fn parse_key(
+        key: &[u8],
+    ) -> std::result::Result<Self, Box<dyn std::error::Error + Send + Sync>> {
         let u32_size = std::mem::size_of::<u32>();
-        let text_id = u32::from_be_bytes(
-            key[0..u32_size]
-                .try_into()
-                .expect("NodeByTextEntry deserialization key was too small (text_id)"),
-        );
+        let text_id = u32::from_be_bytes(key[0..u32_size].try_into()?);
         let mut offset = u32_size;
 
-        let corpus_ref = u32::from_be_bytes(
-            key[offset..(offset + u32_size)]
-                .try_into()
-                .expect("NodeByTextEntry deserialization key was too small (corpus_ref)"),
-        );
+        let corpus_ref = u32::from_be_bytes(key[offset..(offset + u32_size)].try_into()?);
         offset += u32_size;
 
-        let node_id = NodeID::from_be_bytes(
-            key[offset..]
-                .try_into()
-                .expect("NodeByTextEntry deserialization key was too small (node_id)"),
-        );
+        let node_id = NodeID::from_be_bytes(key[offset..].try_into()?);
 
-        NodeByTextEntry {
+        Ok(NodeByTextEntry {
             text_id,
             corpus_ref,
             node_id,
-        }
+        })
     }
 }
 
@@ -377,12 +351,12 @@ where
         // TODO: implement handling the "virtual_tokenization_from_namespace" and "virtual_tokenization_mapping" corpus properties
 
         progress_callback("calculating node statistics (before update)");
-        db.get_node_annos_mut().calculate_statistics();
+        db.get_node_annos_mut().calculate_statistics()?;
 
         db.apply_update(&mut updates, &progress_callback)?;
 
         progress_callback("calculating node statistics (after update)");
-        db.get_node_annos_mut().calculate_statistics();
+        db.get_node_annos_mut().calculate_statistics()?;
 
         for c in db.get_all_components(None, None) {
             progress_callback(&format!("calculating statistics for component {}", c));
@@ -543,7 +517,7 @@ where
                     .filter_map(|key_value| {
                         let splitted: Vec<_> = key_value.splitn(2, ':').collect();
                         if splitted.len() == 2 {
-                            Some((splitted[0].into(), splitted[1].into()))
+                            Some((splitted[0].trim().into(), splitted[1].trim().into()))
                         } else {
                             None
                         }
@@ -949,7 +923,7 @@ where
     Ok(ParsedCorpusTable {
         toplevel_corpus_name: corpus_by_id
             .get(toplevel_corpus_id)
-            .ok_or_else(|| RelAnnisError::CorpusNotFound(*toplevel_corpus_id))?
+            .ok_or(RelAnnisError::CorpusNotFound(*toplevel_corpus_id))?
             .name
             .clone(),
         corpus_by_preorder,
@@ -1030,7 +1004,8 @@ where
     let mut last_textprop: Option<TextProperty> = None;
     let mut last_token: Option<NodeID> = None;
 
-    for (current_textprop, current_token) in token_by_index.try_iter()? {
+    for token in token_by_index.iter()? {
+        let (current_textprop, current_token) = token?;
         // if the last token/text value is valid and we are still in the same text
         if let (Some(last_token), Some(last_textprop)) = (last_token, last_textprop) {
             if last_textprop.corpus_id == current_textprop.corpus_id
@@ -1045,13 +1020,13 @@ where
                 };
                 updates.add_event(UpdateEvent::AddEdge {
                     source_node: id_to_node_name
-                        .try_get(&last_token)?
+                        .get(&last_token)?
                         .ok_or(RelAnnisError::NodeNotFound(last_token))?
-                        .into(),
+                        .to_string(),
                     target_node: id_to_node_name
-                        .try_get(&current_token)?
+                        .get(&current_token)?
                         .ok_or(RelAnnisError::NodeNotFound(current_token))?
-                        .into(),
+                        .to_string(),
                     layer: ordering_layer,
                     component_type: AnnotationComponentType::Ordering.to_string(),
                     component_name: current_textprop.segmentation.clone().into(),
@@ -1076,24 +1051,26 @@ fn add_automatic_cov_edge_for_node(
     let left_pos = load_node_and_corpus_result
         .textpos_table
         .node_to_left
-        .try_get(&n)?
+        .get(&n)?
         .ok_or(RelAnnisError::NoLeftPositionForNode(n))?;
 
     let right_pos = load_node_and_corpus_result
         .textpos_table
         .node_to_right
-        .try_get(&n)?
+        .get(&n)?
         .ok_or(RelAnnisError::NoRightPositionForNode(n))?;
 
     // find left/right aligned basic token
-    let left_aligned_tok = load_node_and_corpus_result
+    let left_aligned_tok: Option<u64> = load_node_and_corpus_result
         .textpos_table
         .token_by_left_textpos
-        .try_get(&left_pos)?;
-    let right_aligned_tok = load_node_and_corpus_result
+        .get(&left_pos)?
+        .map(|v| *v);
+    let right_aligned_tok: Option<u64> = load_node_and_corpus_result
         .textpos_table
         .token_by_right_textpos
-        .try_get(&right_pos)?;
+        .get(&right_pos)?
+        .map(|v| *v);
 
     // If only one of the aligned token is missing, use it for both sides, this is consistent with
     // the relANNIS import of ANNIS3
@@ -1111,12 +1088,12 @@ fn add_automatic_cov_edge_for_node(
     let left_tok_pos = load_node_and_corpus_result
         .textpos_table
         .token_to_index
-        .try_get(&left_aligned_tok)?
+        .get(&left_aligned_tok)?
         .ok_or(RelAnnisError::LeftAlignedNotFound(n))?;
     let right_tok_pos = load_node_and_corpus_result
         .textpos_table
         .token_to_index
-        .try_get(&right_aligned_tok)?
+        .get(&right_aligned_tok)?
         .ok_or(RelAnnisError::RightAlignedNotFound(n))?;
 
     // Create a template TextProperty to which we only change the
@@ -1134,19 +1111,16 @@ fn add_automatic_cov_edge_for_node(
         let tok_id = load_node_and_corpus_result
             .textpos_table
             .token_by_index
-            .try_get(&tok_idx)?
+            .get(&tok_idx)?
             .ok_or_else(|| RelAnnisError::NoTokenForPosition(tok_idx.clone()))?;
-        if n != tok_id {
+        if n != *tok_id {
             let edge = Edge {
                 source: n,
-                target: tok_id,
+                target: *tok_id,
             };
 
             // only add edge if no other coverage edge exists
-            if !load_rank_result
-                .text_coverage_edges
-                .try_contains_key(&edge)?
-            {
+            if !load_rank_result.text_coverage_edges.contains_key(&edge)? {
                 let nodes_with_same_source = (
                     Bound::Included(Edge {
                         source: n,
@@ -1169,24 +1143,24 @@ fn add_automatic_cov_edge_for_node(
                     // Get the original component name for this target node
                     load_rank_result
                         .component_for_parentless_target_node
-                        .try_get(&n)?
+                        .get(&n)?
                         .map_or_else(
                             || (String::default(), String::default()),
-                            |c| (c.layer, c.name),
+                            |c| (c.layer.to_owned(), c.name.to_owned()),
                         )
                 };
 
                 updates.add_event(UpdateEvent::AddEdge {
                     source_node: load_node_and_corpus_result
                         .id_to_node_name
-                        .try_get(&n)?
+                        .get(&n)?
                         .ok_or(RelAnnisError::NodeNotFound(n))?
-                        .into(),
+                        .to_string(),
                     target_node: load_node_and_corpus_result
                         .id_to_node_name
-                        .try_get(&tok_id)?
-                        .ok_or(RelAnnisError::NodeNotFound(tok_id))?
-                        .into(),
+                        .get(&tok_id)?
+                        .ok_or(RelAnnisError::NodeNotFound(*tok_id))?
+                        .to_string(),
                     layer: component_layer.into(),
                     component_type: AnnotationComponentType::Coverage.to_string(),
                     component_name: component_name.into(),
@@ -1210,17 +1184,18 @@ where
     // add explicit coverage edges for each node in the special annis namespace coverage component
     progress_callback("calculating the automatically generated Coverage edges");
 
-    for (n, textprop) in load_node_and_corpus_result
+    for item in load_node_and_corpus_result
         .textpos_table
         .node_to_left
-        .try_iter()?
+        .iter()?
     {
+        let (n, textprop) = item?;
         // Do not calculate automatic coverage edges for token
         if textprop.segmentation.is_empty()
             && !load_node_and_corpus_result
                 .textpos_table
                 .token_to_index
-                .try_contains_key(&n)?
+                .contains_key(&n)?
         {
             if let Err(e) = add_automatic_cov_edge_for_node(
                 updates,
@@ -1254,7 +1229,8 @@ where
     let mut added_whitespace_label_count = 0;
 
     // Iterate over all texts of the graph separately
-    for (text_key, text) in texts.try_iter()? {
+    for text in texts.iter()? {
+        let (text_key, text) = text?;
         let mut text_char_it = text.val.chars();
         let mut current_text_offset = 0;
 
@@ -1278,13 +1254,12 @@ where
             .token_by_index
             .range(min_text_prop..max_text_prop)
             .peekable();
-        while let Some((_, current_token_id)) = token_iterator.next() {
+        while let Some(token) = token_iterator.next() {
+            let (_, current_token_id) = token?;
             // Get the character borders for this token
             if let (Some(left_text_pos), Some(right_text_pos)) = (
-                textpos_table.node_to_left_char.try_get(&current_token_id)?,
-                textpos_table
-                    .node_to_right_char
-                    .try_get(&current_token_id)?,
+                textpos_table.node_to_left_char.get(&current_token_id)?,
+                textpos_table.node_to_right_char.get(&current_token_id)?,
             ) {
                 let token_left_char = left_text_pos.val as usize;
                 let token_right_char = right_text_pos.val as usize;
@@ -1302,9 +1277,9 @@ where
                     }
                     current_text_offset += skipped_before_token;
 
-                    if let Some(token_name) = id_to_node_name.try_get(&current_token_id)? {
+                    if let Some(token_name) = id_to_node_name.get(&current_token_id)? {
                         updates.add_event(UpdateEvent::AddNodeLabel {
-                            node_name: token_name.into(),
+                            node_name: token_name.to_string(),
                             anno_ns: ANNIS_NS.to_string(),
                             anno_name: TOK_WHITESPACE_BEFORE.to_string(),
                             anno_value: covered_text_before,
@@ -1325,9 +1300,9 @@ where
                 // Get the token borders of the next token to determine where the whitespace after this token is
                 // The whitespace end position is non-inclusive.
                 let mut whitespace_end_pos = None;
-                if let Some((_, next_token_id)) = token_iterator.peek() {
+                if let Some(Ok((_, next_token_id))) = token_iterator.peek() {
                     if let Some(next_token_left_pos) =
-                        textpos_table.node_to_left_char.try_get(next_token_id)?
+                        textpos_table.node_to_left_char.get(next_token_id)?
                     {
                         whitespace_end_pos = Some(next_token_left_pos.val as usize);
                     }
@@ -1357,9 +1332,9 @@ where
                         current_text_offset += 1;
                     }
                 }
-                if let Some(token_name) = id_to_node_name.try_get(&current_token_id)? {
+                if let Some(token_name) = id_to_node_name.get(&current_token_id)? {
                     updates.add_event(UpdateEvent::AddNodeLabel {
-                        node_name: token_name.into(),
+                        node_name: token_name.to_string(),
                         anno_ns: ANNIS_NS.to_string(),
                         anno_name: TOK_WHITESPACE_AFTER.to_string(),
                         anno_value: covered_text_after,
@@ -1636,7 +1611,7 @@ where
     textpos_table.token_by_left_textpos.compact()?;
     textpos_table.token_by_right_textpos.compact()?;
 
-    if !(textpos_table.token_by_index.try_is_empty())? {
+    if !(textpos_table.token_by_index.is_empty())? {
         calculate_automatic_token_order(
             updates,
             &textpos_table.token_by_index,
@@ -1684,7 +1659,7 @@ where
         let col_id = get_field_not_null(&line, 0, "id", &node_anno_tab_path)?;
         let node_id: NodeID = col_id.parse()?;
         let node_name = id_to_node_name
-            .try_get(&node_id)?
+            .get(&node_id)?
             .ok_or(RelAnnisError::NodeNotFound(node_id))?;
         let col_ns = get_field(&line, 1, "namespace", &node_anno_tab_path)?.unwrap_or_default();
         let col_name = get_field_not_null(&line, 2, "name", &node_anno_tab_path)?;
@@ -1695,11 +1670,11 @@ where
             // If 'NULL', use an "invalid" string so it can't be found by its value, but only by its annotation name
             let anno_val = &col_val.unwrap_or_else(|| INVALID_STRING.clone());
 
-            if let Some(seg) = missing_seg_span.try_get(&node_id)? {
+            if let Some(seg) = missing_seg_span.get(&node_id)? {
                 // add all missing span values from the annotation, but don't add NULL values
-                if seg == col_name.as_ref() && has_valid_value {
+                if seg.as_str() == col_name.as_str() && has_valid_value {
                     updates.add_event(UpdateEvent::AddNodeLabel {
-                        node_name: node_name.clone().into(),
+                        node_name: node_name.to_string(),
                         anno_ns: ANNIS_NS.to_owned(),
                         anno_name: TOK.to_owned(),
                         anno_value: anno_val.to_string(),
@@ -1708,7 +1683,7 @@ where
             }
 
             updates.add_event(UpdateEvent::AddNodeLabel {
-                node_name: node_name.into(),
+                node_name: node_name.to_string(),
                 anno_ns: col_ns.to_string(),
                 anno_name: col_name.to_string(),
                 anno_value: anno_val.to_string(),
@@ -1861,18 +1836,18 @@ where
 
         if let Some(parent_as_str) = get_field(&line, pos_parent, "parent", &rank_tab_path)? {
             let parent: u32 = parent_as_str.parse()?;
-            if let Some(source) = pre_to_node_id.get(&parent) {
+            if let Some(source) = pre_to_node_id.get(&parent)? {
                 // find the responsible edge database by the component ID
                 if let Some(c) = component_by_id.get(&component_ref) {
                     updates.add_event(UpdateEvent::AddEdge {
                         source_node: id_to_node_name
-                            .try_get(&source)?
-                            .ok_or(RelAnnisError::NodeNotFound(source))?
-                            .into(),
+                            .get(&source)?
+                            .ok_or(RelAnnisError::NodeNotFound(*source))?
+                            .to_string(),
                         target_node: id_to_node_name
-                            .try_get(&target)?
+                            .get(&target)?
                             .ok_or(RelAnnisError::NodeNotFound(target))?
-                            .into(),
+                            .to_string(),
                         layer: c.layer.clone().into(),
                         component_type: c.get_type().to_string(),
                         component_name: c.name.clone().into(),
@@ -1880,7 +1855,10 @@ where
 
                     let pre: u32 = get_field_not_null(&line, 0, "pre", &rank_tab_path)?.parse()?;
 
-                    let e = Edge { source, target };
+                    let e = Edge {
+                        source: *source,
+                        target,
+                    };
 
                     if c.get_type() == AnnotationComponentType::Coverage {
                         load_rank_result
@@ -1943,8 +1921,8 @@ where
         let line = result?;
 
         let pre = get_field_not_null(&line, 0, "pre", &edge_anno_tab_path)?.parse::<u32>()?;
-        if let Some(c) = rank_result.components_by_pre.try_get(&pre)? {
-            if let Some(e) = rank_result.edges_by_pre.try_get(&pre)? {
+        if let Some(c) = rank_result.components_by_pre.get(&pre)? {
+            if let Some(e) = rank_result.edges_by_pre.get(&pre)? {
                 let ns = get_field(&line, 1, "namespace", &edge_anno_tab_path)?.unwrap_or_default();
                 let name = get_field_not_null(&line, 2, "name", &edge_anno_tab_path)?;
                 // If 'NULL', use an "invalid" string so it can't be found by its value, but only by its annotation name
@@ -1953,16 +1931,16 @@ where
 
                 updates.add_event(UpdateEvent::AddEdgeLabel {
                     source_node: id_to_node_name
-                        .try_get(&e.source)?
+                        .get(&e.source)?
                         .ok_or(RelAnnisError::NodeNotFound(e.source))?
-                        .into(),
+                        .to_string(),
                     target_node: id_to_node_name
-                        .try_get(&e.target)?
+                        .get(&e.target)?
                         .ok_or(RelAnnisError::NodeNotFound(e.target))?
-                        .into(),
+                        .to_string(),
                     layer: c.layer.clone().into(),
                     component_type: c.get_type().to_string(),
-                    component_name: c.name.into(),
+                    component_name: c.name.to_string(),
                     anno_ns: ns.to_string(),
                     anno_name: name.to_string(),
                     anno_value: val.to_string(),
@@ -2103,7 +2081,7 @@ fn add_subcorpora(
             let corpus = corpus_table
                 .corpus_by_id
                 .get(corpus_id)
-                .ok_or_else(|| RelAnnisError::CorpusNotFound(*corpus_id))?;
+                .ok_or(RelAnnisError::CorpusNotFound(*corpus_id))?;
 
             let corpus_name = &corpus.name;
 
@@ -2155,7 +2133,8 @@ fn add_subcorpora(
     } // end for each document/sub-corpus
 
     // add a node for each text and the connection between all sub-nodes of the text
-    for (text_key, text) in texts.iter() {
+    for text in texts.iter()? {
+        let (text_key, text) = text?;
         // add text node (including its name)
         if let Some(corpus_ref) = text_key.corpus_ref {
             let text_name = utf8_percent_encode(&text.name, SALT_URI_ENCODE_SET).to_string();
@@ -2187,14 +2166,15 @@ fn add_subcorpora(
                 text_id: text_key.id,
                 node_id: NodeID::max_value(),
             };
-            for (text_entry, _) in node_node_result.nodes_by_text.range(min_key..=max_key) {
+            for item in node_node_result.nodes_by_text.range(min_key..=max_key) {
+                let (text_entry, _) = item?;
                 let n = text_entry.node_id;
                 updates.add_event(UpdateEvent::AddEdge {
                     source_node: node_node_result
                         .id_to_node_name
-                        .try_get(&n)?
+                        .get(&n)?
                         .ok_or(RelAnnisError::NodeNotFound(n))?
-                        .into(),
+                        .to_string(),
                     target_node: text_full_name.clone(),
                     layer: ANNIS_NS.to_owned(),
                     component_type: AnnotationComponentType::PartOf.to_string(),
@@ -2284,6 +2264,40 @@ mod tests {
         .unwrap();
 
         // Check that the node was added to the missing segmentation span map
-        assert_eq!(true, result.missing_seg_span.contains_key(&680));
+        assert_eq!(true, result.missing_seg_span.contains_key(&680).unwrap());
+    }
+
+    /// Regression Test for https://github.com/korpling/graphANNIS/issues/222
+    #[test]
+    fn trim_resolver_mappings() {
+        let resolver_entry =
+            "example	NULL	layer	node	htmldoc	edition	hidden	0	hide_tok:true;annos:abc, def   ; config: edition";
+        // Write resolver_entry to temporary file
+        let parent = tempfile::tempdir().unwrap();
+        let resolver_path = parent.path().join("resolver_vis_map.annis");
+        let mut f = std::fs::File::create(&resolver_path).unwrap();
+        writeln!(f, "{}", resolver_entry).unwrap();
+        f.flush().unwrap();
+
+        // Parse the resolver entry
+        let mut config = CorpusConfiguration::default();
+        load_resolver_vis_map(parent.path(), &mut config, true, &|_| {}).unwrap();
+        // 6 default rules and an additional rule from the file
+        assert_eq!(7, config.visualizers.len());
+
+        assert_eq!("edition", config.visualizers[1].display_name.as_str(),);
+        assert_eq!(3, config.visualizers[1].mappings.len());
+        assert_eq!(
+            Some(&"true".to_string()),
+            config.visualizers[1].mappings.get("hide_tok")
+        );
+        assert_eq!(
+            Some(&"abc, def".to_string()),
+            config.visualizers[1].mappings.get("annos")
+        );
+        assert_eq!(
+            Some(&"edition".to_string()),
+            config.visualizers[1].mappings.get("config")
+        );
     }
 }
